"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var React = require("react");
var R = require("ramda");
var util_1 = require("../../util");
var base_1 = require("../base");
var keyboard_interface_1 = require("./keyboard.interface");
var key_1 = require("./key");
var Keyboard = /** @class */ (function (_super) {
    __extends(Keyboard, _super);
    /**
     * @stable [03.09.2018]
     * @param {IKeyboardProps} props
     */
    function Keyboard(props) {
        var _this = _super.call(this, props) || this;
        _this.onSelect = _this.onSelect.bind(_this);
        _this.state = { position: _this.currentPosition, mode: 0, useUppercase: false };
        return _this;
    }
    /**
     * @stable [03.09.2018]
     */
    Keyboard.prototype.componentDidMount = function () {
        _super.prototype.componentDidMount.call(this);
        util_1.addChild(this.self);
    };
    /**
     * @stable [03.09.2018]
     */
    Keyboard.prototype.componentWillUnmount = function () {
        util_1.removeChild(this.self);
        _super.prototype.componentWillUnmount.call(this);
    };
    /**
     * @stable [08.05.2018]
     * @returns {JSX.Element}
     */
    Keyboard.prototype.render = function () {
        var _this = this;
        var props = this.props;
        var state = this.state;
        var keys = props.layout[state.mode];
        var keysEls = [];
        keys.forEach(function (row, index) {
            keysEls.push(React.createElement("div", { key: "keyboard-row-" + index, className: 'rac-flex rac-flex-center' }, row.map(function (key, index2) { return (React.createElement(key_1.KeyboardKey, { key: "keyboard-row-column-" + index + "-" + index2, value: key, useUppercase: state.useUppercase, onSelect: _this.onSelect })); })));
        });
        // React waiting for a wrapper at least before clearing => <div>KEYBOARD_DIV</div>
        return (React.createElement("div", null,
            React.createElement("div", { ref: 'self', className: 'rac-keyboard rac-absolute-bottom-position rac-no-user-select' },
                React.createElement("div", { className: 'rac-flex' }, this.uiFactory.makeIcon({
                    key: 'keyboard-close-action',
                    type: 'cancel',
                    simple: true,
                    className: 'rac-keyboard-close',
                    onClick: props.onClose,
                })),
                keysEls)));
    };
    /**
     * @stable [08.05.2018]
     * @param {KeyboardKeyT} key
     */
    Keyboard.prototype.onSelect = function (key) {
        var nextValue;
        var props = this.props;
        var state = this.state;
        var jEl = this.jField;
        var position = this.currentPosition;
        var chars = jEl.val().split('');
        var keyAsString = key;
        var keyAsObject = key;
        if (util_1.isString(keyAsString) || keyAsObject.type === keyboard_interface_1.KeyboardKeyEnum.SPACE) {
            var keyValue = util_1.isString(keyAsString) ? keyAsString : keyAsObject.value;
            nextValue = R.insert(position, state.useUppercase ? keyValue.toUpperCase() : keyValue, chars);
            props.onChange(nextValue.join(''));
            this.setState({ position: position + 1 });
        }
        else {
            switch (keyAsObject.type) {
                case keyboard_interface_1.KeyboardKeyEnum.UPPERCASE:
                    this.setState({ useUppercase: !state.useUppercase });
                    break;
                case keyboard_interface_1.KeyboardKeyEnum.CHANGE_LAYOUT:
                    this.setState({ mode: this.state.mode === props.layout.length - 1 ? 0 : this.state.mode + 1 });
                    break;
                case keyboard_interface_1.KeyboardKeyEnum.BACKSPACE:
                    if (position > 0) {
                        nextValue = R.remove(position - 1, 1, chars).join('');
                        props.onChange(nextValue);
                        this.setState({ position: position - 1 });
                    }
                    break;
            }
        }
    };
    Object.defineProperty(Keyboard.prototype, "jField", {
        /**
         * @stable [08.05.2018]
         * @returns {IJqInput}
         */
        get: function () {
            return $(this.props.field);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Keyboard.prototype, "currentPosition", {
        /**
         * @stable [21.08.2018]
         * @returns {number}
         */
        get: function () {
            return this.props.field.value.length;
        },
        enumerable: true,
        configurable: true
    });
    Keyboard.defaultProps = {
        layout: [keyboard_interface_1.KEYBOARD_QWERTY_LAYOUT, keyboard_interface_1.KEYBOARD_QWERTY_DIGITAL_LAYOUT],
    };
    return Keyboard;
}(base_1.BaseComponent));
exports.Keyboard = Keyboard;
//# sourceMappingURL=keyboard.component.js.map